@@ -42,6 +42,7 @@
 #include <QHeaderView>
 #include <QDockWidget>
 #include <QTableView>
+#include <QPrinter>
 #include <KAction>
 #include <KHTMLView>
 #include <KHelpMenu>
@@ -132,7 +133,7 @@ KAlgebra::KAlgebra(QWidget *parent) : KMainWindow(parent)
 	QWidget *console = new QWidget(m_tabs);
 	QVBoxLayout *c_layo = new QVBoxLayout(console);
 	c_results = new ConsoleHtml(console);
-	c_results->view()->setFocusPolicy(Qt::NoFocus);
+	c_results->setFocusPolicy(Qt::NoFocus);
 	c_dock_vars = new QDockWidget(i18n("Variables"), this);
 	c_dock_vars->setFeatures(QDockWidget::DockWidgetFloatable|QDockWidget::DockWidgetMovable);
 	addDockWidget(Qt::RightDockWidgetArea, c_dock_vars);
@@ -154,7 +155,7 @@ KAlgebra::KAlgebra(QWidget *parent) : KMainWindow(parent)
 	
 	m_tabs->addTab(console, i18n("&Console"));
 	console->setLayout(c_layo);
-	c_layo->addWidget(c_results->view());
+	c_layo->addWidget(c_results);
 	c_layo->addWidget(c_exp);
 	
 	connect(c_exp, SIGNAL(returnPressed()), this, SLOT(operate()));
@@ -214,8 +215,9 @@ KAlgebra::KAlgebra(QWidget *parent) : KMainWindow(parent)
 	b_tools->addTab(b_funcs, i18n("List"));
 	
 	b_funced = new FunctionEdit(b_tools);
-	b_funced->setVariables(new Analitza::Variables);
+	b_funced->setVariables(c_varsModel->variables());
 	connect(b_funced, SIGNAL(accept()), this, SLOT(new_func()));
+	connect(b_funced, SIGNAL(removeEditingPlot()), this, SLOT(remove_func()));
 	b_tools->addTab(b_funced, KIcon("list-add"), i18n("&Add"));
 	
 	QTableView* b_varsView=new QTableView(b_tools);
@@ -359,7 +361,9 @@ KAlgebra::KAlgebra(QWidget *parent) : KMainWindow(parent)
 	KHelpMenu* help = new KHelpMenu(this, KGlobal::mainComponent().aboutData());
 	menuBar()->addMenu(help->menu());
 
+#ifdef __GNUC__
 #warning TODO: Port to PlotsModel
+#endif
 // 	connect(b_funcsModel, SIGNAL(functionModified(QString,Analitza::Expression)),
 // 			c_results, SLOT(modifyVariable(QString,Analitza::Expression)));
 // 	connect(b_funcsModel, SIGNAL(functionRemoved(QString)),
@@ -425,6 +429,18 @@ void KAlgebra::new_func()
 	m_graph2d->setFocus();
 }
 
+void KAlgebra::remove_func()
+{
+	Q_ASSERT(b_funced->editing());
+	b_funcsModel->removeRow(b_funcs->currentIndex().row());
+	
+	b_funced->setEditing(false);
+	b_funced->clear();
+	b_tools->setCurrentIndex(0);
+	b_funcs->setCurrentIndex(QModelIndex());
+	m_graph2d->setFocus();
+}
+
 void KAlgebra::edit_func(const QModelIndex &idx)
 {
 	b_tools->setTabText(1, i18n("&Editing"));
@@ -558,10 +574,22 @@ void KAlgebra::set_solid()
 void KAlgebra::save3DGraph()
 {
 #ifdef HAVE_OPENGL
-	QString path = KFileDialog::getSaveFileName(KUrl(), i18n("*.png|PNG File"), this, QString(), KFileDialog::ConfirmOverwrite);
+	QString path = KFileDialog::getSaveFileName(KUrl(), i18n("*.png|PNG File\n*.pdf|PDF Document"), this, QString(), KFileDialog::ConfirmOverwrite);
 	if(!path.isEmpty()) {
 		QPixmap px = m_graph3d->renderPixmap();
-		px.save(path);
+		if(path.endsWith(".pdf")) {
+			QPrinter printer;
+			printer.setOutputFormat(QPrinter::PdfFormat);
+			printer.setOutputFileName(path);
+			printer.setPaperSize(px.size(), QPrinter::DevicePixel);
+			printer.setPageMargins(0,0,0,0, QPrinter::DevicePixel);
+			QPainter painter;
+			painter.begin(&printer);
+			painter.drawPixmap(QPoint(0,0), px);
+			painter.end();
+		} else {
+			px.save(path);
+		}
 	}
 #endif
 }
